
import flask
import json

from pipguitool.PIP import PIP

from flask import Blueprint, request, abort, render_template

import os
import argparse


# Args
# spark-submit arguments
parser = argparse.ArgumentParser()
parser.add_argument("-p", "--port", help="Port (default: 4040)", type=int, default=4040)
parser.add_argument("-b", "--prefix", help="Path Prefix ex. /pipui", default=None)
parser.add_argument("-i", "--path", help="Target path for pip when installing a new package", default=None)
args = parser.parse_args()


p = PIP(install_path=args.path)

# BluePrint
bp = Blueprint('bp', __name__, static_folder= 'static', template_folder='templates')
# APP
app = flask.Flask(__name__, static_url_path= '/static' if args.prefix is None else '{}/static'.format(args.prefix) )
app.config["DEBUG"] = True
app.config['UPLOAD_FOLDER'] = '/tmp/uploads'
# if upload folder does not exist then
if not os.path.exists(app.config['UPLOAD_FOLDER']):
    os.makedirs(app.config['UPLOAD_FOLDER'])


@bp.route('/list', methods=['GET'])
def list():
	
    return json.dumps(p.list())


@bp.route('/install', methods=['POST'])
def install():

	if not request.json or not 'packages' in request.json or not hasattr(request.json['packages'], "append"):
		abort(400)
	
	return json.dumps(p.install(request.json['packages'])), 201


@bp.route('/remove', methods=['POST'])
def remove():

	if not request.json or not 'packages' in request.json or not hasattr(request.json['packages'], "append"):
		abort(400)
	
	return json.dumps(p.remove(request.json['packages'])), 201


@bp.route('/file', methods=['POST'])
def file():

	_file = request.files.get('file')
	
	# Save file to UPLOAD dir
	filePath = os.path.join(app.config['UPLOAD_FOLDER'], _file.filename)
	_file.save(filePath)

	# Install the whl file
	res = p.installWhl(filePath)

	# remove the uploaded file
	os.remove(filePath)

	return json.dumps(res), 201


@bp.route('/', methods=['GET'])
def index():

	return render_template('index.html', packages=p.list(), prefix= '/' if args.prefix is None else '{}/'.format(args.prefix) )



app.register_blueprint(bp, url_prefix= '/' if args.prefix is None else args.prefix )




def main():

	app.run(host="0.0.0.0", port=args.port, threaded=True)

if __name__ == '__main__':
    main()





from subprocess import Popen, PIPE
import json

from wheel_inspect import inspect_wheel

import re


# REGEX package name
_p_regex = re.compile('^[a-zA-Z0-9\-]+([=]{2}[0-9]+([\.][0-9]+([\.][0-9]+)?)?)?$')
_file_regex = re.compile('^[a-zA-Z0-9-\_\.]+$')


class PIP:

	def __init__(self, install_path=None):
		
		self.install_path = install_path

	def command(self, command):

		if self.install_path is None:
			return ["pip", command]
		else:
			return ["pip", command, "-t", self.install_path]


	def install(self, packages):

		pkgs = {}
		for pkg in packages:

			# validate package name
			if not self.validate(pkg):
				pkgs[pkg] = { 'name': pkg, 'installed': False, 'message': '\'{}\' is invalid'.format(pkg) }
				continue
			
			name = pkg.split("==")[0]
			p = Popen(self.command("install")+[pkg], stdin=PIPE, stdout=PIPE, stderr=PIPE)
			output, err = p.communicate(b"input data that is passed to subprocess' stdin")
			pkgs[name] = { 'name': name, 'installed': True if p.returncode == 0 else False, 'message': (output if p.returncode == 0 else err).decode("utf-8") }

		l = self.list()
		for pkg in l:
			if pkg['name'] in pkgs.keys():
				pkgs[pkg['name']]['version'] = pkg['version']


		return pkgs

	def installWhl(self, filePath):

		filename = filePath.split("/")[-1]

		try:

			# validate filename
			if not self.validate_filename(filename):
				raise Exception("Invalide filename")

			package_info = inspect_wheel(filePath)['dist_info']['metadata']

			p = Popen(self.command("install")+[filePath], stdin=PIPE, stdout=PIPE, stderr=PIPE)
			output, err = p.communicate(b"input data that is passed to subprocess' stdin")
			#(output if p.returncode == 0 else err).decode("utf-8")
			if p.returncode == 0:
				output = {**{"message": output.decode("utf-8")}, **{"name": package_info['name'], "version": package_info['version'], "author": package_info['author']}, **{"installed": True}}
			else:
				output = {**{"message": err.decode("utf-8")}, **{"filename": filename}, **{"installed": False}}

		except:
			output = {"filename": filename, "installed": False, "message": "Invalid File"}

		return [output]

	def remove(self, packages):

		pkgs = {}
		for pkg in packages:
			name = pkg.split("==")[0]
			p = Popen(["pip", "uninstall", "-y", pkg], stdin=PIPE, stdout=PIPE, stderr=PIPE)
			output, err = p.communicate(b"input data that is passed to subprocess' stdin")
			pkgs[name] = { 'name': name, 'removed': True if p.returncode == 0 else False, 'message': (output if p.returncode == 0 else err).decode("utf-8") }

		return pkgs

	def getPackagesInfo(self, packages):

		pkgs = {}
		p = Popen(["pip", "show"]+packages, stdin=PIPE, stdout=PIPE, stderr=PIPE)
		output, err = p.communicate(b"input data that is passed to subprocess' stdin")

		if p.returncode == 0:
			allpkgs = map(lambda x: x.strip(), output.decode("utf-8").split("---"))
			for pkg in allpkgs:
				properties = list(map(lambda line: list(map(lambda x: x.strip(), line.split(":", 1))), pkg.split("\n")))
				properties = { line[0].lower(): line[1] for line in properties}
				if len(properties) <= 0:
					continue
				pkgs[properties['name']] = properties

		return pkgs

	def list(self):

		p = Popen(["pip", "list", "--format", "json"], stdin=PIPE, stdout=PIPE, stderr=PIPE)
		output, err = p.communicate(b"input data that is passed to subprocess' stdin")

		return [ {"name": e['name'], "version": e['version'], "message": "available"} for e in json.loads(output) ] if p.returncode == 0 else []

	def validate(self, package):

		return not _p_regex.match(package) is None

	def validate_filename(self, filename):

		return not _file_regex.match(filename) is None

	def has(self, package):

		return len(list(filter(lambda x: x['name'] == package, self.list()))) > 0


def main():
    #print(PIP().list())
    result = PIP().install(['scipy', 'numpy==1.18', 'asdasfafg3&_'])
    for i in result:
    	if not result[i]['installed']:
    		print(i, result[i]['message'])
    #print(PIP().installWhl("../dist/pip_gui_tools-0.0.8-py3-none-any.whl"))
    #print(PIP().installWhl("../dist/invalid_filename&.ad"))
    #print(PIP().remove(['scipy', 'numpy', 'asdqsd']))
    #print(PIP().getPackagesInfo(['scipy', 'numpy', 'asdqsd']))

if __name__ == '__main__':
    main()
	